'use strict';

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _nodeBanner = _interopRequireDefault(require("node-banner"));

var _projectConfig = _interopRequireDefault(require("../../utils/projectConfig"));

var _messages = require("../../utils/messages");

var _directoryPrompt = _interopRequireDefault(require("../../utils/directoryPrompt"));

var _exec = _interopRequireDefault(require("../../utils/exec"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var vuexStoreTemplate = _fs["default"].readFileSync(_path["default"].resolve(__dirname, '..', '..', 'templates/vuex/store.js'), 'utf8');
/**
 * Choose additional deps to install on the go
 *
 * @returns {Promise<void>}
 */


var addDeps = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(deps, _ref) {
    var dev, templateDir, _yield$dirOfChoice, _appData, template, installFlag, vuexNuxtStoreTemplate, vuetifyConfig, nuxtConfig, indexOfLink, config, blankLineIndex, routerIndex, _config, preIndex;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            dev = _ref.dev;
            _context.next = 3;
            return (0, _nodeBanner["default"])('MEVN CLI', 'Light speed setup for MEVN stack based apps.');

          case 3:
            (0, _messages.checkIfConfigFileExists)(); // Global reference to the directory of choice

            templateDir = 'client'; // Get to know whether the deps are to be installed for client/server directory

            if (!_fs["default"].existsSync('./server')) {
              _context.next = 10;
              break;
            }

            _context.next = 8;
            return (0, _directoryPrompt["default"])();

          case 8:
            _yield$dirOfChoice = _context.sent;
            templateDir = _yield$dirOfChoice.dir;

          case 10:
            if (!(templateDir === 'server' && deps.some(function (plugin) {
              return ['vuex', 'vuetify'].includes(plugin);
            }))) {
              _context.next = 12;
              break;
            }

            return _context.abrupt("return");

          case 12:
            _appData = (0, _projectConfig["default"])(), template = _appData.template; // Vuetify bindings for Nuxt-js

            if (template === 'Nuxt-js' && !dev) deps.push('@nuxtjs/vuetify');
            installFlag = dev ? '--save-dev' : '--save'; // Install the opted deps

            _context.next = 17;
            return (0, _exec["default"])("npm install ".concat(installFlag, " ").concat(deps.join(' ')), "Installing ".concat(deps.join(', ')), 'Done', {
              cwd: templateDir
            });

          case 17:
            if (!dev) {
              _context.next = 19;
              break;
            }

            return _context.abrupt("return");

          case 19:
            if (template === 'Nuxt-js') {
              // vuex-store template content for Nuxt-js
              vuexNuxtStoreTemplate = ['export const state = () => ({', '  counter: 0', '})', '', 'export const mutations = {', '  increment (state) {', '\tstate.counter++', '  }', '}']; // Vuetify-Config to be inserted

              vuetifyConfig = ['  /*', '   ** Set up vuetify', '   */', '  buildModules: [', '    // Simple usage', "    '@nuxtjs/vuetify',", '', '    // With options', '    [', "      '@nuxtjs/vuetify',", '      {', '        /* module options */', '      }', '    ]', '  ],']; // Configure vuex-store for Nuxt-js template

              if (deps.includes('vuex')) {
                // Navigate to the store directory and create a basic store template file
                _fs["default"].writeFileSync('./client/store/index.js', vuexNuxtStoreTemplate.join('\n'));
              } // Configure @nuxtjs/vuetify


              if (deps.includes('vuetify')) {
                // Read initial content from nuxt.config.js
                nuxtConfig = _fs["default"].readFileSync('./nuxt.config.js', 'utf8').toString().split('\n'); // Find the position of link within header information

                indexOfLink = nuxtConfig.indexOf(nuxtConfig.find(function (line) {
                  return line.includes('link');
                })); // Insert the respective content

                vuetifyConfig.forEach(function (item, i) {
                  return nuxtConfig.splice(indexOfLink + i + 2, 0, item);
                }); // Write back the updated content

                _fs["default"].writeFileSync('./client/nuxt.config.js', nuxtConfig.join('\n'));
              }
            } else {
              // Configure vuex-store
              if (deps.includes('vuex')) {
                config = _fs["default"].readFileSync('./client/src/main.js', 'utf8').toString().split('\n'); // Creates a new store.js file within the client/src directory.

                _fs["default"].writeFileSync('./client/src/store.js', vuexStoreTemplate); // Fetch the index corresponding to the very first blank line


                blankLineIndex = config.indexOf(config.find(function (line) {
                  return line === '';
                })); // Inserting the import statement for vuex-store

                config.splice(blankLineIndex, 0, "import store from \"./store\";"); // Fetching the position where in which router is passed on to the Vue instance

                routerIndex = config.indexOf(config.find(function (line) {
                  return line.trim() === 'router,';
                })); // Insert store just after router so that it gets passed on to the Vue instance

                config.splice(routerIndex + 1, 0, "  store,"); // Cleaning up

                if (deps.includes('vuetify')) config.splice(blankLineIndex + 1, 1); // Write back the updated config

                _fs["default"].writeFileSync('./client/src/main.js', config.join('\n'));
              } // Configure vuetify


              if (deps.includes('vuetify')) {
                _config = _fs["default"].readFileSync('./client/src/main.js', 'utf8').toString().split('\n'); // Import Vuetify and minified css towards the top of the config file

                ["import Vuetify from 'vuetify';", "import 'vuetify/dist/vuetify.min.css';"].forEach(function (item, i) {
                  return _config.splice(i + 1, 0, item);
                }); // Fetch the index after which the respective config should come up

                preIndex = _config.indexOf(_config.find(function (line) {
                  return line.includes('Vue.config.productionTip');
                })); // Inserting the respective Vuetify config

                ['Vue.use(Vuetify);', ''].forEach(function (item, i) {
                  return _config.splice(preIndex + i, 0, item);
                }); // Write back the updated config

                _fs["default"].writeFileSync('./client/src/main.js', _config.join('\n'));
              }
            }

          case 20:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function addDeps(_x, _x2) {
    return _ref2.apply(this, arguments);
  };
}();

module.exports = addDeps;