'use strict';

var _chalk = _interopRequireDefault(require("chalk"));

var _execa = _interopRequireDefault(require("execa"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _inquirer = _interopRequireDefault(require("inquirer"));

var _nodeBanner = _interopRequireDefault(require("node-banner"));

var _validateNpmPackageName = _interopRequireDefault(require("validate-npm-package-name"));

var _fs2 = _interopRequireDefault(require("../../utils/fs"));

var _constants = require("../../utils/constants");

var _messages = require("../../utils/messages");

var _spinner = _interopRequireDefault(require("../../utils/spinner"));

var _validate = require("../../utils/validate");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var projectPathRelative;
var projectConfig = {};
/**
 * Creates an initial local commit
 *
 * @returns {Void}
 */

var makeInitialCommit = function makeInitialCommit() {
  // Commands to be executed serially
  var commands = ['init', 'add .', "commit -m \"Init\" -m \"MEVN-CLI\""]; // Execute commands serially

  commands.forEach(function (cmd) {
    return _execa["default"].sync('git', cmd.split(' '), {
      cwd: projectPathRelative
    });
  });
};
/**
 * Logs the further actions to be performed
 *
 * @returns {Void}
 */


var showInstructions = function showInstructions() {
  var isCurrentDir = projectPathRelative === '.';

  var userCommandInstruction = _chalk["default"].green.bold('mevn serve');

  if (!isCurrentDir) {
    userCommandInstruction = "".concat(_chalk["default"].green.bold("cd ".concat(projectPathRelative)), " && ").concat(userCommandInstruction);
  }

  console.log();
  console.log();
  console.log(_chalk["default"].cyan.bold("You're all set"));
  console.log(_chalk["default"].cyan.bold("Now, just type in ".concat(userCommandInstruction)));
  var removeCmd = _constants.isWin ? 'rmdir /s /q' : 'rm -rf';

  _execa["default"].shellSync("".concat(removeCmd, " ").concat(_path["default"].join(projectPathRelative, '.git')));

  makeInitialCommit();
};
/**
 * Fetch the boilerplate template of choice
 *
 * @param {String} template - The branch which corresponds to the respective boilerplate template
 * @returns {Promise<void>}
 */


var fetchTemplate = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(templateBranch) {
    var repoUrl, fetchSpinner, _yield$inquirer$promp, requirePwaSupport, configFile, _yield$inquirer$promp2, mode, _configFile, index, _yield$inquirer$promp3, requireServer, serverDir, serverPath, source, dest, renameFromPath, renameToPath;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, _validate.validateInstallation)('git help -g');

          case 2:
            // Boilerplate templates are available within a single repository
            repoUrl = 'https://github.com/madlabsinc/mevn-starter-templates';
            fetchSpinner = new _spinner["default"]('Fetching the boilerplate template');
            fetchSpinner.start();
            _context.prev = 5;
            _context.next = 8;
            return (0, _execa["default"])('git', ['clone', repoUrl, '--branch', templateBranch, '--single-branch', projectPathRelative]);

          case 8:
            _context.next = 14;
            break;

          case 10:
            _context.prev = 10;
            _context.t0 = _context["catch"](5);
            fetchSpinner.fail('Something went wrong');
            throw _context.t0;

          case 14:
            fetchSpinner.stop();

            _fs["default"].writeFileSync("./".concat(projectPathRelative, "/.mevnrc"), JSON.stringify(projectConfig, null, 2)); // Prompt the user whether he/she requires pwa support


            if (!(templateBranch === 'nuxt')) {
              _context.next = 27;
              break;
            }

            _context.next = 19;
            return _inquirer["default"].prompt([{
              name: 'requirePwaSupport',
              type: 'confirm',
              message: 'Do you require pwa support'
            }]);

          case 19:
            _yield$inquirer$promp = _context.sent;
            requirePwaSupport = _yield$inquirer$promp.requirePwaSupport;

            // Write to .mevnrc in order to keep track while installing dependencies
            if (requirePwaSupport) {
              configFile = JSON.parse(_fs["default"].readFileSync("./".concat(projectPathRelative, "/.mevnrc")));
              configFile = _objectSpread({}, configFile, {
                isPwa: true,
                isPwaConfigured: false
              });

              _fs["default"].writeFileSync("./".concat(projectPathRelative, "/.mevnrc"), JSON.stringify(configFile, null, 2));
            } // Choose between Universal/SPA mode


            _context.next = 24;
            return _inquirer["default"].prompt([{
              name: 'mode',
              type: 'list',
              message: 'Choose your preferred mode',
              choices: ['Universal', 'SPA']
            }]);

          case 24:
            _yield$inquirer$promp2 = _context.sent;
            mode = _yield$inquirer$promp2.mode;

            // Update the config file (nuxt.config.js)
            if (mode === 'Universal') {
              _configFile = _fs["default"].readFileSync("./".concat(projectPathRelative, "/client/nuxt.config.js"), 'utf8').toString().split('\n');
              index = _configFile.indexOf(_configFile.find(function (line) {
                return line.includes('mode');
              }));
              _configFile[index] = " mode: 'universal',";

              _fs["default"].writeFileSync("./".concat(projectPathRelative, "/client/nuxt.config.js"), _configFile.join('\n'));
            }

          case 27:
            _context.next = 29;
            return _inquirer["default"].prompt({
              name: 'requireServer',
              type: 'confirm',
              message: 'Do you require server side template (Express.js)'
            });

          case 29:
            _yield$inquirer$promp3 = _context.sent;
            requireServer = _yield$inquirer$promp3.requireServer;

            // Copy server side template files to the destination as required
            if (requireServer) {
              // Configure path
              serverDir = templateBranch === 'graphql' ? 'GraphQL' : 'basic';
              serverPath = ['templates', 'server', serverDir];
              source = _path["default"].join.apply(_path["default"], [__dirname, '..', '..'].concat(serverPath));
              dest = _path["default"].resolve(projectPathRelative); // Copy server template directory to the destination

              (0, _fs2["default"])(source, dest); // Rename the resultant directory to server

              renameFromPath = _path["default"].join(dest, serverDir);
              renameToPath = _path["default"].join(dest, 'server');

              _fs["default"].renameSync(renameFromPath, renameToPath);
            } // Show up initial instructions to the user


            showInstructions();

          case 33:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[5, 10]]);
  }));

  return function fetchTemplate(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Bootstrap a basic MEVN stack based webapp
 *
 * @param {String} appName - Name of the project
 * @returns {Promise<void>}
 */


var initializeProject = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(appName) {
    var hasMultipleProjectNameArgs, isCurrentDir, validationResult, _yield$inquirer$promp4, template, templateBranch;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return (0, _nodeBanner["default"])('MEVN CLI', 'Light speed setup for MEVN stack based apps.');

          case 2:
            hasMultipleProjectNameArgs = process.argv[4] && !process.argv[4].startsWith('-');
            isCurrentDir = false;

            if (appName === '.') {
              isCurrentDir = true;
              appName = _path["default"].basename(process.cwd());
            } // Validation for multiple directory names


            if (hasMultipleProjectNameArgs) {
              (0, _messages.hasStrayArgs)();
            }

            validationResult = (0, _validateNpmPackageName["default"])(appName);

            if (!validationResult.validForNewPackages) {
              (0, _messages.invalidProjectName)(appName);
            }

            if (isCurrentDir) {
              if (_fs["default"].readdirSync('.').length) {
                console.log();
                console.log(_chalk["default"].red.bold("It seems the current directory isn't empty."));
                console.log();
                process.exit(0);
              }
            }

            if (!isCurrentDir && _fs["default"].existsSync(appName)) {
              (0, _messages.directoryExistsInPath)(appName);
            }

            if (_fs["default"].existsSync('./.mevnrc')) {
              console.log();
              console.log(_chalk["default"].cyan.bold(" It seems that you're already within a valid MEVN stack based project"));
              process.exit(0);
            }

            projectPathRelative = isCurrentDir ? '.' : appName;
            _context2.next = 14;
            return _inquirer["default"].prompt([{
              name: 'template',
              type: 'list',
              message: 'Please select your template of choice',
              choices: ['basic', 'pwa', 'graphql', 'Nuxt-js']
            }]);

          case 14:
            _yield$inquirer$promp4 = _context2.sent;
            template = _yield$inquirer$promp4.template;
            projectConfig['name'] = appName;
            projectConfig['template'] = template; // Holds GitHub repo branch corresponding to the respective boilerplate template

            templateBranch = template;

            if (template === 'basic') {
              templateBranch = 'master';
            } else if (template === 'Nuxt-js') {
              templateBranch = 'nuxt';
            }

            fetchTemplate(templateBranch);

          case 21:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function initializeProject(_x2) {
    return _ref2.apply(this, arguments);
  };
}();

module.exports = initializeProject;