'use strict';

var _chalk = _interopRequireDefault(require("chalk"));

var _execa = _interopRequireDefault(require("execa"));

var _fs = _interopRequireDefault(require("fs"));

var _inquirer = _interopRequireDefault(require("inquirer"));

var _projectConfig = _interopRequireDefault(require("../../utils/projectConfig"));

var _validate = require("../../utils/validate");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * Creates a new Heroku app
 *
 * @param {String} dir - Directory of choice
 * @returns {Promise<void>}
 */
var createHerokuApp = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(dir) {
    var _yield$inquirer$promp, appName, stderr;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return _inquirer["default"].prompt({
              name: 'appName',
              type: 'input',
              message: 'Enter a name for the app',
              validate: _validate.validateInput
            });

          case 2:
            _yield$inquirer$promp = _context.sent;
            appName = _yield$inquirer$promp.appName;
            _context.prev = 4;
            _context.next = 7;
            return _execa["default"].shell("heroku create ".concat(appName), {
              cwd: dir
            });

          case 7:
            _context.next = 17;
            break;

          case 9:
            _context.prev = 9;
            _context.t0 = _context["catch"](4);
            stderr = _context.t0.stderr;
            console.log();
            console.log(_chalk["default"].red(stderr));
            console.log();
            _context.next = 17;
            return createHerokuApp(dir);

          case 17:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[4, 9]]);
  }));

  return function createHerokuApp(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Checks whether the user is logged in on Heroku
 *
 * @returns {Boolean}
 */


var isLoggedIn = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.prev = 0;
            _context2.next = 3;
            return _execa["default"].shell('heroku whoami');

          case 3:
            return _context2.abrupt("return", true);

          case 6:
            _context2.prev = 6;
            _context2.t0 = _context2["catch"](0);
            return _context2.abrupt("return", false);

          case 9:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, null, [[0, 6]]);
  }));

  return function isLoggedIn() {
    return _ref3.apply(this, arguments);
  };
}();
/**
 * Deploy the webapp to Heroku
 *
 * @param {String} templateDir - client/server
 * @returns {Promise<void>}
 */


var deployToHeroku = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(templateDir) {
    var projectConfig, template, isPwa, fileContent, _yield$execa$shell, _stdout, staticConfig, starterSource, pkgJson, buildCmd, postInstallScript, _yield$execa$shell2, stdout, _yield$execa$shell3, _stdout2, _yield$inquirer$promp2, uri;

    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return (0, _validate.validateInstallation)('heroku');

          case 2:
            _context3.next = 4;
            return (0, _validate.validateInstallation)('git help -g');

          case 4:
            projectConfig = (0, _projectConfig["default"])();
            template = projectConfig.template;
            isPwa = projectConfig.hasOwnProperty('isPwa') && projectConfig.isPwa;

            if (_fs["default"].existsSync("./".concat(templateDir, "/.git"))) {
              _context3.next = 14;
              break;
            }

            _context3.next = 10;
            return _execa["default"].shell('git init', {
              cwd: templateDir
            });

          case 10:
            fileContent = _fs["default"].existsSync('./server/.env') ? 'node_modules\n.env' : 'node_modules';

            _fs["default"].writeFileSync("./".concat(templateDir, "/.gitignore"), fileContent);

            _context3.next = 19;
            break;

          case 14:
            _context3.next = 16;
            return _execa["default"].shell('git status', {
              cwd: templateDir
            });

          case 16:
            _yield$execa$shell = _context3.sent;
            _stdout = _yield$execa$shell.stdout;

            if (_stdout.includes('nothing to commit')) {
              console.error('No changes detected!');
              process.exit(1);
            }

          case 19:
            // Necessary configurations that are specific to the client side
            if (templateDir === 'client') {
              staticConfig = {
                root: 'dist',
                clean_urls: true,
                routes: {
                  '/**': 'index.html'
                }
              };

              if (!_fs["default"].existsSync('./client/static.json')) {
                _fs["default"].writeFileSync('./client/static.json', JSON.stringify(staticConfig, null, 2));
              }

              starterSource = ['const express = require("express");', 'const serveStatic = require("serve-static");', 'const path = require("path");', 'const app = express();', 'app.use(serveStatic(path.join(__dirname, "dist")));', 'const port = process.env.PORT || 80;', 'app.listen(port);'];
              pkgJson = JSON.parse(_fs["default"].readFileSync('./client/package.json'));
              buildCmd = "npm run ".concat(template === 'Nuxt-js' ? 'generate' : 'build');
              postInstallScript = "if test \"$NODE_ENV\" = \"production\" ; then ".concat(buildCmd, " ; fi "); // eslint-disable-line

              pkgJson = _objectSpread({}, pkgJson, {
                scripts: _objectSpread({}, pkgJson.scripts, {
                  postinstall: postInstallScript,
                  start: 'node server.js'
                })
              });

              if (isPwa) {
                pkgJson.scripts['preinstall'] = 'npm install --save @nuxtjs/pwa';
              }

              if (!_fs["default"].existsSync('./client/server.js')) {
                _fs["default"].writeFileSync('./client/server.js', starterSource.join('\n'));

                pkgJson.scripts['preinstall'] = 'npm install --save express serve-static';

                _fs["default"].writeFileSync('./client/package.json', JSON.stringify(pkgJson, null, 2));
              }
            }

            _context3.next = 22;
            return isLoggedIn();

          case 22:
            if (_context3.sent) {
              _context3.next = 25;
              break;
            }

            _context3.next = 25;
            return _execa["default"].shell('heroku login', {
              stdio: 'inherit',
              cwd: templateDir
            });

          case 25:
            _context3.next = 27;
            return _execa["default"].shell('git remote', {
              cwd: templateDir
            });

          case 27:
            _yield$execa$shell2 = _context3.sent;
            stdout = _yield$execa$shell2.stdout;

            if (stdout.includes('heroku')) {
              _context3.next = 32;
              break;
            }

            _context3.next = 32;
            return createHerokuApp(templateDir);

          case 32:
            if (!(templateDir === 'server' && _fs["default"].existsSync('./server/models'))) {
              _context3.next = 44;
              break;
            }

            _context3.next = 35;
            return _execa["default"].shell('heroku config', {
              cwd: templateDir
            });

          case 35:
            _yield$execa$shell3 = _context3.sent;
            _stdout2 = _yield$execa$shell3.stdout;

            if (_stdout2.includes('DB_URL')) {
              _context3.next = 44;
              break;
            }

            _context3.next = 40;
            return _inquirer["default"].prompt({
              type: 'password',
              name: 'uri',
              message: 'Please provide the path to a cloud based MongoDB URI',
              validate: _validate.validateInput
            });

          case 40:
            _yield$inquirer$promp2 = _context3.sent;
            uri = _yield$inquirer$promp2.uri;
            _context3.next = 44;
            return _execa["default"].shell("heroku config:set DB_URL=".concat(uri), {
              cwd: templateDir
            });

          case 44:
            _context3.next = 46;
            return _execa["default"].shell('git add .', {
              cwd: templateDir
            });

          case 46:
            _context3.next = 48;
            return _execa["default"].shell("git commit -m \"Add files\"", {
              cwd: templateDir
            });

          case 48:
            _context3.next = 50;
            return _execa["default"].shell('git push heroku master', {
              stdio: 'inherit',
              cwd: templateDir
            });

          case 50:
            _context3.next = 52;
            return _execa["default"].shell('heroku open', {
              cwd: templateDir
            });

          case 52:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function deployToHeroku(_x2) {
    return _ref4.apply(this, arguments);
  };
}();

module.exports = deployToHeroku;