'use strict';

var _execa = _interopRequireDefault(require("execa"));

var _fs = _interopRequireDefault(require("fs"));

var _exec = _interopRequireDefault(require("../../utils/exec"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * Adds PWA support to Nuxt-js boilerplate template
 *
 * @returns {Promise<void>}
 */
var configurePwaSupport = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var configFile, nuxtConfigFile, moduleDefStartsWithIndex, contentToAppend;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            configFile = JSON.parse(_fs["default"].readFileSync('./.mevnrc'));

            if (!(configFile['isPwa'] && !configFile['isPwaConfigured'])) {
              _context.next = 13;
              break;
            }

            _context.next = 4;
            return (0, _exec["default"])('npm install --save @nuxtjs/pwa', 'Installing Nuxt.js pwa module', 'Done', {
              cwd: 'client'
            });

          case 4:
            nuxtConfigFile = _fs["default"].readFileSync('client/nuxt.config.js').toString().split('\n'); // Get the index where module definition starts

            moduleDefStartsWithIndex = nuxtConfigFile.indexOf(nuxtConfigFile.find(function (line) {
              return line.includes('modules: [');
            }));
            nuxtConfigFile[moduleDefStartsWithIndex] = "  modules: [";
            contentToAppend = ["  '@nuxtjs/eslint-module',", "  '@nuxtjs/pwa'", '  ],']; // Make use of the updated value

            moduleDefStartsWithIndex++; //  Update modules array to include pwa module

            contentToAppend.forEach(function (config, idx) {
              return nuxtConfigFile.splice(moduleDefStartsWithIndex + idx, 0, config);
            }); // Write back the updated config

            _fs["default"].writeFileSync('client/nuxt.config.js', nuxtConfigFile.join('\n')); // set isPwaConfigured key in the config file to true


            configFile.isPwaConfigured = true;

            _fs["default"].writeFileSync("./.mevnrc", JSON.stringify(configFile, null, 2));

          case 13:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function configurePwaSupport() {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Serve the webapp locally
 *
 * @param {String} projectTemplate - Boilerplate template of choice
 * @param {String} templateDir - Choose between client/server side
 * @returns {Promise<void>}
 */


var serveProject = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(projectTemplate, templateDir) {
    var port;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (templateDir === 'client') {
              port = projectTemplate === 'Nuxt-js' ? '3000' : '3002';
            } else {
              port = projectTemplate === 'graphql' ? '9000/graphql' : '9000/api';
            }

            if (_fs["default"].existsSync("./".concat(templateDir, "/node_modules"))) {
              _context2.next = 4;
              break;
            }

            _context2.next = 4;
            return (0, _exec["default"])('npm install', 'Installing dependencies in the background. Hold on...', 'Dependencies were successfully installed', {
              cwd: templateDir
            });

          case 4:
            if (!(projectTemplate === 'Nuxt-js')) {
              _context2.next = 7;
              break;
            }

            _context2.next = 7;
            return configurePwaSupport();

          case 7:
            _execa["default"].shell("npm run serve -- --port ".concat(port, " --open"), {
              stdio: 'inherit',
              cwd: templateDir
            });

          case 8:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function serveProject(_x, _x2) {
    return _ref2.apply(this, arguments);
  };
}();

module.exports = serveProject;